@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2009, 2014 IBM Corp.
+ * Copyright (c) 2009, 2016 IBM Corp.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -14,6 +14,8 @@
  *    Ian Craggs, Allan Stockdill-Mander - initial implementation
  *    Ian Craggs - fix for bug #409702
  *    Ian Craggs - allow compilation for OpenSSL < 1.0
+ *    Ian Craggs - fix for bug #453883
+ *    Ian Craggs - fix for bug #480363, issue 13
  *******************************************************************************/
 
 /**
@@ -294,7 +296,7 @@ int pem_passwd_cb(char* buf, int size, int rwflag, void* userdata)
 	{
 		strncpy(buf, (char*)(userdata), size);
 		buf[size-1] = '\0';
-		rc = strlen(buf);
+		rc = (int)strlen(buf);
 	}
 	FUNC_EXIT_RC(rc);
 	return rc;
@@ -353,7 +355,6 @@ void SSL_destroy_mutex(ssl_mutex_type* mutex)
 	rc = CloseHandle(*mutex);
 #else
 	rc = pthread_mutex_destroy(mutex);
-	free(mutex);
 #endif
 	FUNC_EXIT_RC(rc);
 }
@@ -382,10 +383,13 @@ extern unsigned long SSLThread_id(void)
 
 extern void SSLLocks_callback(int mode, int n, const char *file, int line)
 {
-	if (mode & CRYPTO_LOCK)
-		SSL_lock_mutex(&sslLocks[n]);
-	else
-		SSL_unlock_mutex(&sslLocks[n]);
+	if (sslLocks)
+	{
+		if (mode & CRYPTO_LOCK)
+			SSL_lock_mutex(&sslLocks[n]);
+		else
+			SSL_unlock_mutex(&sslLocks[n]);
+	}
 }
 
 int SSLSocket_initialize()   
@@ -442,7 +446,19 @@ exit:
 void SSLSocket_terminate()
 {
 	FUNC_ENTRY;
-	free(sslLocks);
+	EVP_cleanup();
+	ERR_free_strings();
+	CRYPTO_set_locking_callback(NULL);
+	if (sslLocks)
+	{
+		int i = 0;
+
+		for (i = 0; i < CRYPTO_num_locks(); i++)
+		{
+			SSL_destroy_mutex(&sslLocks[i]);
+		}
+		free(sslLocks);
+	}
 	FUNC_EXIT;
 }
 
@@ -628,7 +644,7 @@ exit:
  *  @param actual_len the actual number of bytes read
  *  @return completion code
  */
-char *SSLSocket_getdata(SSL* ssl, int socket, int bytes, int* actual_len)
+char *SSLSocket_getdata(SSL* ssl, int socket, size_t bytes, size_t* actual_len)
 {
 	int rc;
 	char* buf;
@@ -642,7 +658,7 @@ char *SSLSocket_getdata(SSL* ssl, int socket, int bytes, int* actual_len)
 
 	buf = SocketBuffer_getQueuedData(socket, bytes, actual_len);
 
-	if ((rc = SSL_read(ssl, buf + (*actual_len), (size_t)(bytes - (*actual_len)))) < 0)
+	if ((rc = SSL_read(ssl, buf + (*actual_len), (int)(bytes - (*actual_len)))) < 0)
 	{
 		rc = SSLSocket_error("SSL_read - getdata", ssl, socket, rc);
 		if (rc != SSL_ERROR_WANT_READ && rc != SSL_ERROR_WANT_WRITE)
@@ -714,9 +730,9 @@ int SSLSocket_putdatas(SSL* ssl, int socket, char* buf0, size_t buf0len, int cou
 	int sslerror;
 
 	FUNC_ENTRY;
-	iovec.iov_len = buf0len;
+	iovec.iov_len = (ULONG)buf0len;
 	for (i = 0; i < count; i++)
-		iovec.iov_len += buflens[i];
+		iovec.iov_len += (ULONG)buflens[i];
 
 	ptr = iovec.iov_base = (char *)malloc(iovec.iov_len);  
 	memcpy(ptr, buf0, buf0len);
